"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PayloadSeal = exports.schema = void 0;
const c5_envelope_1 = require("c5-envelope");
exports.schema = 'https://github.com/mabels/kleisimo/blob/main/schema/encrypted.ts';
class PayloadSeal {
    constructor(key) {
        this.key = key;
    }
    seal({ message, reason }) {
        const jsonHash = toDataJson({ message });
        const hash = jsonHash.hash || '';
        const encrypted = {
            keyId: this.key.id,
            encryptionMethod: this.key.alg,
            message: this.key.encrypt(jsonHash.jsonStr),
            reason,
            hash,
        };
        return c5_envelope_1.Convert.toPayloadT(JSON.stringify({ data: encrypted, kind: exports.schema }));
    }
    unseal(encrypted) {
        if (encrypted.kind === exports.schema) {
            return {
                kind: exports.schema,
                data: Object.assign(Object.assign({}, encrypted.data), { message: JSON.parse(this.key.decrypt(encrypted.data.message).toString()) }),
            };
        }
        return undefined;
    }
}
exports.PayloadSeal = PayloadSeal;
function toDataJson({ message, salt, jsonProps, }) {
    const dataJsonStrings = [];
    const dataHashC = new c5_envelope_1.HashCollector();
    const dataJsonC = new c5_envelope_1.JsonCollector((part) => dataJsonStrings.push(part), jsonProps);
    dataHashC.append({ val: new c5_envelope_1.PlainValType(salt || '') });
    (0, c5_envelope_1.sortKeys)(message, (sval) => {
        dataHashC.append(sval);
        dataJsonC.append(sval);
    });
    return {
        jsonStr: dataJsonStrings.join(''),
        hash: dataHashC.digest(),
    };
}
//# sourceMappingURL=encrypt.js.map