"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SymetricKey = exports.bs58 = void 0;
const crypto = __importStar(require("crypto"));
const base_x_1 = __importDefault(require("base-x"));
exports.bs58 = (0, base_x_1.default)('123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz');
class SymetricKey {
    constructor(prop) {
        this.id = prop.id;
        this.hashSeed = prop.hashSeed || crypto.randomUUID();
        this.alg = prop.alg || 'chacha20-poly1305';
        this.keyBuffer = prop.key ? Buffer.from(prop.key, 'base64') : undefined;
        this.nounceBuffer = prop.nounce ? Buffer.from(prop.nounce, 'base64') : undefined;
        this.keyLength = prop.keyLength || 32;
        this.nounceLength = prop.nounceLength || 12;
    }
    static create(props) {
        return __awaiter(this, void 0, void 0, function* () {
            return new SymetricKey(props).create();
        });
    }
    create() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((rs, rj) => {
                crypto.randomBytes(this.keyLength, (err, key) => {
                    if (err) {
                        rj(err);
                        return;
                    }
                    crypto.randomBytes(this.nounceLength, (err, nounce) => {
                        if (err) {
                            rj(err);
                            return;
                        }
                        this.keyBuffer = key;
                        this.nounceBuffer = nounce;
                        const hash = crypto.createHash('sha256');
                        hash.update(key);
                        hash.update(nounce);
                        this.id = hash.digest().toString('base64');
                        rs(this);
                    });
                });
            });
        });
    }
    get key() {
        if (!this.keyBuffer) {
            throw Error('key is not set!');
        }
        const key = this.keyBuffer.toString('base64');
        return key;
    }
    get nounce() {
        if (!this.nounceBuffer) {
            throw Error('key is not set!');
        }
        const nounce = this.nounceBuffer.toString('base64');
        return nounce;
    }
    decrypt(dec) {
        if (!dec.startsWith('C')) {
            throw Error(`Unknown cipher:${dec}`);
        }
        dec = dec.slice('C'.length);
        const decipher = crypto.createDecipheriv(this.alg, Buffer.from(this.key, 'base64'), Buffer.from(this.nounce, 'base64'), {
            authTagLength: 4,
        });
        const ret = decipher.update(exports.bs58.decode(dec));
        decipher.final();
        return ret;
    }
    encrypt(enc, addition) {
        const cipher = crypto.createCipheriv(this.alg, Buffer.from(this.key, 'base64'), Buffer.from(this.nounce, 'base64'), {
            authTagLength: 4,
        });
        let out;
        if (typeof enc === 'string') {
            out = Buffer.from(enc);
        }
        else {
            out = enc;
        }
        if (addition) {
            cipher.setAAD(addition, { plaintextLength: addition === null || addition === void 0 ? void 0 : addition.length });
        }
        const ret = cipher.update(out);
        cipher.final();
        // C means chacha20-poly1305
        return 'C' + exports.bs58.encode(ret);
    }
    hash(enc) {
        const hasher = crypto.createHash('sha256');
        hasher.update(this.hashSeed);
        hasher.update(enc.toUpperCase().replace(/[\n\r \t]+/g, ''));
        // 'A' means sha256 upcase und replace
        return 'A' + exports.bs58.encode(hasher.digest());
    }
}
exports.SymetricKey = SymetricKey;
//# sourceMappingURL=key.js.map