/// <reference types="node" />
import { SymetricKey } from './key';
export declare type Obj<T> = T;
export interface DecodeObj {
    e: string[];
    h: string;
}
export interface ValueType<T, H = string> {
    readonly name: string;
    readonly type: string;
    key?: SymetricKey;
    set(t: T): T;
    get(): T;
    setKleisimo(t: H): T;
    getKleisimo(): H;
}
export declare class AttributeRegister<O, H> {
    readonly attributes: ValueType<unknown, unknown>[];
    register<T, H>(vt: ValueType<T, H>): void;
    asUnencrypted(): O;
    asEncrypted(): H;
}
export declare class Attribute<T> implements ValueType<T> {
    readonly name: string;
    readonly type: string;
    key: SymetricKey;
    value: T;
    constructor(ar: AttributeRegister<unknown, unknown>, name: string, type: string, key: SymetricKey);
    set(t: T): T;
    get(): T;
    unpad(b: string): string;
    decode(b: string): DecodeObj;
    setKleisimo(t: string): T;
    pad(p: string): Buffer;
    encode(o: DecodeObj): string;
    getKleisimo(): string;
}
export interface DateObj {
    month: number;
    year: number;
    encrypted: string;
}
export declare class DateAttribute implements ValueType<Date, DateObj> {
    readonly name: string;
    readonly type: string;
    key: SymetricKey;
    value: Date;
    constructor(ar: AttributeRegister<unknown, unknown>, name: string, key: SymetricKey);
    get(): Date;
    setKleisimo(t: DateObj): Date;
    getKleisimo(): DateObj;
    set(t: Date): Date;
}
export interface ZipObj {
    readonly zip: string;
}
export declare class ZipAttribute implements ValueType<string, ZipObj> {
    readonly name: string;
    readonly type: string;
    key: SymetricKey;
    value: string;
    constructor(ar: AttributeRegister<unknown, unknown>, name: string, key: SymetricKey);
    get(): string;
    setKleisimo(t: ZipObj): string;
    getKleisimo(): ZipObj;
    set(t: string): string;
}
export declare class OpenAttribute<T> implements ValueType<T, T> {
    readonly name: string;
    readonly type: string;
    value: T;
    constructor(ar: AttributeRegister<unknown, unknown>, name: string, type: string);
    key?: SymetricKey | undefined;
    set(t: T): T;
    get(): T;
    getKleisimo(): T;
    setKleisimo(t: T): T;
}
