"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OpenAttribute = exports.ZipAttribute = exports.DateAttribute = exports.Attribute = exports.AttributeRegister = void 0;
// export interface RegisteredValueType<T> extends ValueType<T> {
//     readonly name: string;
//     readonly type: string;
//     readonly key: Key;
// }
class AttributeRegister {
    constructor() {
        this.attributes = [];
    }
    // protected readonly key: Key;
    // constructor(key: Key) {
    //     this.key = key;
    // }
    register(vt) {
        this.attributes.push(vt);
    }
    asUnencrypted() {
        return this.attributes.reduce((ret, a) => {
            ret[a.name] = a.get();
            return ret;
        }, {});
    }
    asEncrypted() {
        return this.attributes.reduce((ret, a) => {
            ret[a.name] = a.getKleisimo();
            return ret;
        }, {});
    }
}
exports.AttributeRegister = AttributeRegister;
class Attribute {
    constructor(ar, name, type, key) {
        this.name = name;
        this.type = type;
        this.value = undefined;
        this.key = key;
        ar.register(this);
    }
    set(t) {
        this.value = t;
        return t;
    }
    get() {
        return this.value;
    }
    unpad(b) {
        return this.key.decrypt(b).toString('utf-8').split('\x00')[0];
    }
    decode(b) {
        const o = b.split('-');
        return {
            e: o.slice(1),
            h: o[0],
        };
    }
    setKleisimo(t) {
        const o = this.decode(t);
        const x = this.unpad(o.e[0]);
        // unpad
        switch (this.type) {
            case 'number':
                this.value = parseInt(x);
                break;
            case 'string':
                this.value = x;
                break;
            default:
                throw Error(`unknown type: ${this.type}`);
        }
        return this.value;
    }
    pad(p) {
        let padlen = p.length;
        if (p.length % 8 !== 0) {
            padlen += 8 - (p.length % 8);
        }
        const o = Buffer.alloc(padlen, 0);
        o.write(p, 'utf-8');
        return o;
    }
    encode(o) {
        return [o.h, ...o.e].join('-');
    }
    getKleisimo() {
        const v = '' + this.value;
        // pad
        return this.encode({
            e: [this.key.encrypt(this.pad(v))],
            h: this.key.hash(v),
        });
    }
}
exports.Attribute = Attribute;
class DateAttribute {
    constructor(ar, name, key) {
        this.name = name;
        this.type = 'Date';
        this.value = undefined;
        this.key = key;
        ar.register(this);
    }
    get() {
        return this.value;
    }
    setKleisimo(t) {
        this.value = new Date(this.key.decrypt(t.encrypted).toString());
        return this.value;
    }
    getKleisimo() {
        return {
            month: this.value.getMonth(),
            year: this.value.getFullYear(),
            encrypted: this.key.encrypt(this.value.toISOString()),
        };
    }
    set(t) {
        this.value = t;
        return t;
    }
}
exports.DateAttribute = DateAttribute;
class ZipAttribute {
    constructor(ar, name, key) {
        this.name = name;
        this.type = 'string';
        this.value = undefined;
        this.key = key;
        ar.register(this);
    }
    get() {
        return this.value;
    }
    setKleisimo(t) {
        this.value = t.zip;
        return this.value;
    }
    getKleisimo() {
        return {
            zip: this.value,
        };
    }
    set(t) {
        this.value = t;
        return t;
    }
}
exports.ZipAttribute = ZipAttribute;
class OpenAttribute {
    constructor(ar, name, type) {
        this.value = undefined;
        this.name = name;
        this.type = type;
        ar.register(this);
    }
    set(t) {
        this.value = t;
        return t;
    }
    get() {
        return this.value;
    }
    getKleisimo() {
        return this.value;
    }
    setKleisimo(t) {
        this.value = t;
        return this.value;
    }
}
exports.OpenAttribute = OpenAttribute;
//# sourceMappingURL=kleisimo.js.map