"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const kleisimo_1 = require("./kleisimo");
const key_1 = require("./key");
function keymap(entry) {
    {
        return {
            name: entry.name,
            type: entry.type,
            key: { key: entry.key.key, hashSeed: entry.key.hashSeed, nounce: entry.key.nounce },
        };
    }
}
class MockValueType {
    constructor(name) {
        this.name = 'name';
        this.type = 'string';
        this.name = name;
    }
    set(t) {
        throw new Error('Method not implemented.');
    }
    get() {
        throw new Error('Method not implemented.');
    }
    setKleisimo(t) {
        throw new Error('Method not implemented.');
    }
    getKleisimo() {
        throw new Error('Method not implemented.');
    }
}
test('AttributeRegister', () => {
    const ar = new kleisimo_1.AttributeRegister();
    expect(ar.attributes).toEqual([]);
    ar.register(new MockValueType('1'));
    expect(ar.attributes).toEqual([{ name: '1', type: 'string' }]);
    ar.register(new MockValueType('2'));
    expect(ar.attributes).toEqual([
        { name: '1', type: 'string' },
        { name: '2', type: 'string' },
    ]);
});
test('Open', () => {
    const ar = new kleisimo_1.AttributeRegister();
    const op = new kleisimo_1.OpenAttribute(ar, 'bla', 'test');
    expect(ar.attributes).toEqual([{ name: 'bla', type: 'test', value: undefined }]);
    expect(op.get()).toEqual(undefined);
    op.set('in');
    expect(op.get()).toEqual('in');
    expect(op.getKleisimo()).toEqual('in');
    op.setKleisimo('bla');
    expect(op.get()).toEqual('bla');
    expect(op.getKleisimo()).toEqual('bla');
});
test('KleisimoZipAttribute', () => __awaiter(void 0, void 0, void 0, function* () {
    const ar = new kleisimo_1.AttributeRegister();
    const key = new key_1.SymetricKey({
        hashSeed: 'hash',
        nounce: Buffer.alloc(12, 0xff).toString('base64'),
        key: Buffer.alloc(32, 0x01).toString('base64'),
    });
    yield key.create();
    const op = new kleisimo_1.ZipAttribute(ar, 'bla', key);
    expect(ar.attributes.map(keymap)).toEqual([
        { name: 'bla', type: 'string', value: undefined, key: { hashSeed: 'hash', key: op.key.key, nounce: op.key.nounce } },
    ]);
    expect(op.get()).toEqual(undefined);
    op.set('in');
    expect(op.get()).toEqual('in');
    expect(op.getKleisimo()).toEqual({ zip: 'in' });
    op.setKleisimo({ zip: 'bla' });
    expect(op.get()).toEqual('bla');
    expect(op.getKleisimo()).toEqual({ zip: 'bla' });
}));
test('KleisimoDateAttribute', () => {
    const ar = new kleisimo_1.AttributeRegister();
    const key = new key_1.SymetricKey({
        hashSeed: 'hash',
        nounce: Buffer.alloc(12, 0xff).toString('base64'),
        key: Buffer.alloc(32, 0x01).toString('base64'),
    });
    const op = new kleisimo_1.DateAttribute(ar, 'bla', key);
    expect(ar.attributes.map(keymap)).toEqual([
        { name: 'bla', type: 'Date', value: undefined, key: { hashSeed: 'hash', key: op.key.key, nounce: op.key.nounce } },
    ]);
    expect(op.get()).toEqual(undefined);
    const now = new Date();
    op.set(now);
    expect(op.get()).toEqual(now);
    expect(op.getKleisimo()).toEqual({
        encrypted: key.encrypt(now.toISOString()),
        month: now.getMonth(),
        year: now.getFullYear(),
    });
    op.setKleisimo({
        month: 44,
        year: 44,
        encrypted: key.encrypt(now.toISOString()),
    });
    expect(op.get().toISOString()).toEqual(now.toISOString());
    expect(op.getKleisimo()).toEqual({
        encrypted: key.encrypt(now.toISOString()),
        month: now.getMonth(),
        year: now.getFullYear(),
    });
});
test('KleisimoAttribute<string>', () => {
    const ar = new kleisimo_1.AttributeRegister();
    const key = new key_1.SymetricKey({
        hashSeed: 'hash',
        nounce: Buffer.alloc(12, 0xff).toString('base64'),
        key: Buffer.alloc(32, 0x01).toString('base64'),
    });
    const op = new kleisimo_1.Attribute(ar, 'bla', 'string', key);
    expect(ar.attributes.map(keymap)).toEqual([
        { name: 'bla', type: 'string', value: undefined, key: { hashSeed: 'hash', key: op.key.key, nounce: op.key.nounce } },
    ]);
    expect(op.get()).toEqual(undefined);
    op.set('bla');
    expect(op.get()).toEqual('bla');
    expect(op.getKleisimo()).toEqual(op.encode({
        e: [key.encrypt(op.pad(op.get()))],
        h: key.hash(op.get()),
        // t: 'string',
    }));
    op.setKleisimo(op.encode({
        e: [key.encrypt('xxxxx')],
        h: key.hash('yyyyy'),
        // t: 'string',
    }));
    expect(op.get()).toEqual('xxxxx');
    expect(op.getKleisimo()).toEqual(op.encode({
        e: [key.encrypt(op.pad('xxxxx'))],
        h: key.hash('xxxxx'),
        // t: 'string',
    }));
});
test('KleisimoAttribute<number>', () => {
    const ar = new kleisimo_1.AttributeRegister();
    const key = new key_1.SymetricKey({
        hashSeed: 'hash',
        nounce: Buffer.alloc(12, 0xff).toString('base64'),
        key: Buffer.alloc(32, 0x01).toString('base64'),
    });
    const op = new kleisimo_1.Attribute(ar, 'bla', 'number', key);
    expect(ar.attributes.map(keymap)).toEqual([
        { name: 'bla', type: 'number', value: undefined, key: { hashSeed: 'hash', key: op.key.key, nounce: op.key.nounce } },
    ]);
    expect(op.get()).toEqual(undefined);
    op.set(4711);
    expect(op.get()).toEqual(4711);
    expect(op.getKleisimo()).toEqual(op.encode({
        e: [key.encrypt(op.pad('' + op.get()))],
        h: key.hash('' + op.get()),
        // t: 'number',
    }));
    op.setKleisimo(op.encode({
        e: [key.encrypt('4711')],
        h: key.hash('yyyyy'),
        // t: 'number',
    }));
    expect(op.get()).toEqual(4711);
    expect(op.getKleisimo()).toEqual(op.encode({
        e: [key.encrypt(op.pad('4711'))],
        h: key.hash('4711'),
        // t: 'number',
    }));
});
//# sourceMappingURL=kleisimo.test.js.map